
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#ifndef _NETWORKMANAGER_H_
#define _NETWORKMANAGER_H_

#include <QtCore/QObject>
#include <QtNetwork/QHostAddress>

class QTimer;
class QUdpSocket;

class NetworkManager :
    public QObject
{
    Q_OBJECT

public:

    enum NetworkMode
    {
        MODE_SERVER = 0,
        MODE_CLIENT
    };

    NetworkManager(QObject *parent = 0);
    virtual ~NetworkManager();

    int getPort() const {return m_udpPort; }
    int getMode() const {return m_mode; }
    const QString & getServerName() const {return m_serverName; }
    QAbstractSocket::SocketState getState() const;
    int getClientNb() const {return m_clients.size(); }

signals:

    void serverAlive();
    void serverDown();
    void newMessage(const QString&);

public slots:

    void start();
    void setServerName(const QString &name);
    void setPort(int port);
    void setMode(int mode);

    void searchServer(const QString &hostname);
    void sendNewMessage(const QString &);
    void saveSettings();

private slots:

    void broadcastDatagramSearchServers(const QHostAddress &addr);
    void broadcastDatagramSendServerIdentification(const QHostAddress &adr);
    void broadcastDatagramSendBackground(const QHostAddress &addr);
    void processPendingDatagrams();
    void pingServer();

private:

    void updateConfig();
    void loadSettings();
    void initialize();

    QUdpSocket *m_udpSocket;
    QTimer *m_timer;
    QTimer *m_pingTimer;
    int m_udpPort;
    QString m_serverName;
    int m_mode;
    QString m_desc;
    QString m_currentMessage;
    QTimer *m_waitAnswer;
    QList<QHostAddress> m_clients;
    bool m_initialized;
};

#endif // _NETWORKMANAGER_H_
