
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include "NetworkManagerConfigWidget.h"
#include "NetworkManager.h"
#include "ThreadPool.h"

#include "ui_NetworkManagerConfigWidget.h"

NetworkManagerConfigWidget::NetworkManagerConfigWidget(QWidget *parent)
    : QWidget(parent),
    ui(new Ui::NetworkManagerConfigWidget)
{
    ui->setupUi(this);

    m_manager = new NetworkManager();

    m_manager->moveToThread( ThreadPool::getInstance()->getThread() );

    // transmit NetworkManager signals
    connect( m_manager, SIGNAL( serverAlive() ), this, SIGNAL( newState() ) );
    connect( m_manager, SIGNAL( serverAlive() ), this, SLOT( serverConnected() ) );
    connect( m_manager, SIGNAL( serverDown() ), this, SIGNAL( newState() ) );
    connect( m_manager, SIGNAL( serverDown() ), this, SLOT( serverDisconnected() ) );
    connect( m_manager, SIGNAL( newMessage(const QString &) ), this, SIGNAL( newMessage(const QString &) ) );

    connect( this, SIGNAL( sigSearchServer(const QString &) ), m_manager, SLOT( searchServer(const QString &) ) );
    connect( this, SIGNAL( sigSetPort(int) ), m_manager, SLOT( setPort(int) ) );
    connect( this, SIGNAL( sigSetServerName(const QString &) ), m_manager, SLOT( setServerName(const QString &) ) );
    connect( this, SIGNAL( sigSetMode(int) ), m_manager, SLOT( setMode(int) ) );
    connect( this, SIGNAL( sigSendNewMessage(const QString &) ), m_manager, SLOT( sendNewMessage(const QString &) ) );
    connect( this, SIGNAL( sigSaveSetting() ), m_manager, SLOT( saveSettings() ) );
    connect( this, SIGNAL( sigStart() ), m_manager, SLOT( start() ) );

    updateUI();
}

NetworkManagerConfigWidget::~NetworkManagerConfigWidget()
{
    delete m_manager;
    delete ui;
}

void NetworkManagerConfigWidget::updateUI()
{
    ui->groupBoxServer->setChecked(m_manager->getMode() == NetworkManager::MODE_CLIENT);
    ui->lineEditServerName->setText( m_manager->getServerName() );
    ui->lineEditServerPort->setText( QString::number( m_manager->getPort() ) );
}

void NetworkManagerConfigWidget::on_pushButtonTestConnection_pressed()
{
    QString serverName = ui->lineEditServerName->text();
    int udpPort = ui->lineEditServerPort->text().toInt();

    if (serverName.isEmpty() || !udpPort)
        return;

    emit sigSetPort(udpPort);

    // clear the previous text
    ui->labelTestConnection->setText("");
    emit sigSearchServer(serverName);
}

void NetworkManagerConfigWidget::on_lineEditServerName_returnPressed()
{
    // change color to valid modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::black);
    ui->lineEditServerName->setPalette(pal);

    emit sigSetServerName( ui->lineEditServerName->text() );
}

void NetworkManagerConfigWidget::on_lineEditServerPort_returnPressed()
{
    // change color to valid modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::black);
    ui->lineEditServerPort->setPalette(pal);

    emit sigSetPort( ui->lineEditServerPort->text().toInt() );
}

void NetworkManagerConfigWidget::serverConnected()
{
    ui->labelTestConnection->setText( tr("Server %1 reached ...").arg( m_manager->getServerName() ) );
    emit synchronizeByNetwork(true);
}

void NetworkManagerConfigWidget::serverDisconnected()
{
    ui->labelTestConnection->setText( tr("Disconnection from server %1 ...").arg( m_manager->getServerName() ) );
}

void NetworkManagerConfigWidget::on_groupBoxServer_clicked(bool val)
{
    bool synchroServer = val;

    // update the network manager
    if (synchroServer)
    {
        emit sigSetMode(NetworkManager::MODE_CLIENT);
        on_pushButtonTestConnection_pressed(); // check connection with server
    }
    else
    {
        emit sigSetMode(NetworkManager::MODE_SERVER);
    }

    emit synchronizeByNetwork(synchroServer);
    emit newState();
}

void NetworkManagerConfigWidget::on_lineEditServerName_textEdited(const QString & /*text*/)
{
    // change color to highlight modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::red);
    ui->lineEditServerName->setPalette(pal);
}

void NetworkManagerConfigWidget::on_lineEditServerPort_textEdited(const QString & /*text*/)
{
    // change color to highlight modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::red);
    ui->lineEditServerPort->setPalette(pal);
}

void NetworkManagerConfigWidget::sendNewMessage(const QString &mess)
{
    emit sigSendNewMessage(mess);
}

void NetworkManagerConfigWidget::saveSettings()
{
    m_manager->saveSettings();
}

int NetworkManagerConfigWidget::getNetworkMode() const
{
    return m_manager->getMode();
}

int NetworkManagerConfigWidget::getClientNb() const
{
    return m_manager->getClientNb();
}

QAbstractSocket::SocketState NetworkManagerConfigWidget::getState() const
{
    return m_manager->getState();
}

void NetworkManagerConfigWidget::start()
{
    emit sigStart();
}
