
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#ifndef _SYNCWALL_H_
#define _SYNCWALL_H_

#include <QtCore/QStringList>
#include <QtCore/QDir>
#include <QtCore/QVector>
#include <QtCore/QTimer>

#include <QtGui/QMainWindow>
#include <QtGui/QSystemTrayIcon>
#include <QtGui/QItemSelection>

class QTextEdit;
class QListWidgetItem;

// forward declarations
class Scheduler;
class WallpaperProvider;
#ifndef Q_WS_PM // OS/2
class QxtGlobalShortcut;
#endif
class ListImagesModel;

namespace Ui
{
class SyncWallClass;
}

class SyncWall :   public QMainWindow
{
    Q_OBJECT

public:

    enum typState
    {
        STATE_STANDARD = 0, // scheduling on, no connected client
        STATE_IDLE, // scheduling off, no connected client
        STATE_SERVER, // connected client
        STATE_CLIENT_CONNECTED, // connected to server
        STATE_CLIENT_DISCONNECTED, // can't connect to server
        STATE_CLIENT_INVALID // connected to server but invalid protocol (version ?)
    };

    explicit SyncWall(QWidget *parent = 0, Qt::WFlags flags = 0);
    ~SyncWall();

signals:

    void sigSaveSettings();

private slots:

    void on_pushButtonQuit_pressed();
    void on_pushButtonHelp_pressed();

    void on_pushButtonAddDirectory_pressed();
    void on_pushButtonAddFiles_pressed();
    void on_pushButtonRefresh_pressed();
    void on_pushButtonDeleteImage_pressed();
    void on_pushButtonClearLog_pressed();
    void on_pushButtonSaveLog_pressed();
    void on_pushButtonSetWallpaper_pressed();

    void on_actionPreviewImage_triggered();
    void on_actionSetWallpaper_triggered();
    void on_actionDeleteImage_triggered();

    void on_pushButtonPreview_pressed();
    void on_groupBoxHotKey_clicked(bool val);

    void on_checkBoxRandomOrder_stateChanged ( int );
    void on_actionShowUI_triggered();

    // preferences

    void closeEvent(QCloseEvent *event);

    void iconActivated(QSystemTrayIcon::ActivationReason reason);

    void printToLog(const QString & mess);

    void selectionChanged(const QItemSelection&, const QItemSelection&);
    void changeWallpaper(int index = -1, bool manual = false);

    // slot for network
    void searchForWallpaper(const QString &);

    void checkForBlinking();
    void blink();

    void checkState();

    void on_widgetKeySequenceNext_keySequenceChanged(const QKeySequence &seq);
    void on_widgetKeySequencePrev_keySequenceChanged(const QKeySequence &seq);
    void on_widgetKeySequenceOpen_keySequenceChanged(const QKeySequence &seq);
    void on_widgetKeySequenceOffset_keySequenceChanged(const QKeySequence &seq);
    void on_checkBoxStartInTray_stateChanged ( int state );
    void on_checkBoxSplashscreenAtStartup_stateChanged ( int state );
    void on_checkBoxSplashscreenTransparentBackground_stateChanged ( int state );
    void on_checkBoxStartWithSystem_stateChanged ( int state );

    // view list/icon/...
    //void on_pushButtonViewList_pressed();
    void on_pushButtonViewDetail_pressed();
    void on_pushButtonViewThumbnail_pressed();
    void on_actionQuit_activated();
    void on_actionNextWallpaper_activated();
    void on_actionPreviousWallpaper_activated();

    void dropEvent(QDropEvent *event);
    void dragEnterEvent(QDragEnterEvent *event);

    void keyPressEvent(QKeyEvent* event);

    void on_listViewFiles_customContextMenuRequested(const QPoint & pos);
    void on_treeViewFiles_customContextMenuRequested(const QPoint & pos);
    void on_treeViewFiles_doubleClicked(const QModelIndex & index);
    void on_listViewFiles_doubleClicked(const QModelIndex & index);

    // language
    void on_comboBoxLanguage_activated(int);

    // prefs
    void on_pushButtonResetPreferences_pressed();

    void on_pushButtonResetCache_pressed();

    void wakeUp(const QString &mess);

    void saveSettings();

    void computeIndexList();

    void on_pushButtonUserMove_pressed();
    void on_pushButtonUserValid_pressed();
    void on_pushButtonUserCancel_pressed();

    void on_tabWidgetMain_currentChanged(int);
    void on_tabWidgetPreferences_currentChanged(int);

    void checkIfUserCanMove();
    void changeMonitorSupport();

    void on_listWidgetTranslations_itemClicked( QListWidgetItem * item);

private:

    void updateUi();
    void updateAbout();
    void updateInfosImageFormats();
    void updateInfosLanguages();
    void updatePrefsLanguage();

    bool eventFilter(QObject* watched, QEvent* event);

    // setup
    void setupGlobalKeys();

    // settings

    void loadSettings();

    // tray icon
    void createTrayIcon();
    void createActionsSystemTray();

    QStringList getSubDirectoryList(const QDir & dir);
    void refreshWallpaperProviders(bool force = false);
    void updateDisplayedFileList(WallpaperProvider *ws);

    int getRandomIndex(int max);

    void updateApplicationIdentity();
    QString availablesImageFormats();
    QStringList availablesImageFormatsList();

    void changeTrayIcon();

    int getState();
    void initStateDefs();

    void updateCredits();
    void updateChangelog();
    void fillTextedit(const QString &file, QTextEdit *);
    void restart();

    void addDirToProviders(const QString &,bool,bool);

    void checkForSmallInterface();
    void enableBrowsing(bool);

    // index

    int getNextImageIndex();
    int getPrevImageIndex();

    void trackFileInView(int ind);

    Ui::SyncWallClass *ui;

    // global key variables
    bool m_useGlobalHotKey;
    QString m_globalHotkeyNext;
    QString m_globalHotkeyPrev;
    QString m_globalHotkeyOpen;
    QString m_globalHotkeyOffset;
#ifndef Q_WS_PM // OS/2
    QxtGlobalShortcut *m_shortCutNext;
    QxtGlobalShortcut *m_shortCutPrev;
    QxtGlobalShortcut *m_shortCutOpen;
    QxtGlobalShortcut *m_shortCutOffset;
#endif

    bool m_firstCloseMessage;

    QString m_appName;
    QString m_version;
    QString m_title;

    QString m_lastDirectory;
    QString m_lastFile;

    //QStringList m_listFiles;

    //system tray
    QSystemTrayIcon *m_trayIcon;
    QMenu *m_trayIconMenu;

    Scheduler *m_scheduler;
    int m_logLevel;
    bool m_randomOrderChange;

    // preferences

    QList<WallpaperProvider *> m_currentProviders;

    // display
    int m_currentImageIndex;

    QString m_currentLanguage;
    QTimer m_blinkTimer;
    bool m_blinking;

    QMap<int, QString> m_mapToStateDesc;
    QMap<int, QString> m_mapToStateIcon;

    bool m_startInTray;
    bool m_splashscreenAtStartup;
    bool m_splashscreenTransparentBackground;
    bool m_startWithSystem;

    ListImagesModel *m_listModel;
    bool m_resetConfig;

    QVector<int> m_listIndex;
    int m_currentListIndex;
    QString m_currPreviewImage;
};

#endif // _SYNCWALL_H_
