
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#ifndef _SYSTEMINTEGRATION_H_
#define _SYSTEMINTEGRATION_H_

#include <QtCore/QObject>
#include <QtGui/QImage>
#include <QtCore/QProcess>
#include <QtCore/QStringList>
#include <QtCore/QQueue>
#include <QtCore/QMap>

typedef struct
{
    QString command;
    QStringList args;
}  JobDef;

class SystemIntegration :
    public QObject
{
    Q_OBJECT

public:

// enum Desktop type
    enum desktopType
    {
        DESKTOP_UNKNOWN = 0,
        DESKTOP_KDE3,
        DESKTOP_KDE4,
        DESKTOP_GNOME,
        DESKTOP_GNOME3,
        DESKTOP_XFCE,
        DESKTOP_XFCE4,
        DESKTOP_FLUXBOX,
        DESKTOP_ICEWM,
        DESKTOP_BLACKBOX,
        DESKTOP_OPENBOX,
        DESKTOP_LXDE,
        DESKTOP_WINDOWMAKER,
        DESKTOP_ENLIGHTENMENT,
        DESKTOP_FVWM, // end Unix desktop
        DESKTOP_WINDOWS, // XP, VISTA, SEVEN
        DESKTOP_MACOSX, // MAC OSX
        DESKTOP_OS2, // OS/2
        DESKTOP_TYPE_NUMBER
    };

    SystemIntegration(QObject *parent = 0);
    ~SystemIntegration();

    QString desktopToString(int val);
    int detectCurrentDesktop();

    QString getDesktopTips(int);

    void setDesktopType(int type)
    {
        m_desktopType = type;
    }

    int getDesktopType() const
    {
        return m_desktopType;
    }

public slots:

    void applyImage(const QList<QImage> &);

//! process errors  from QProcess (python script)
    void errorProcess( QProcess::ProcessError error);

//! read the std output errors (output from the script running)
    void readStandardError();

//! read the std output (output from the script running)
    void readStandardOutput();

//! called at the end of the script
    void finishedProcess( int exitCode, QProcess::ExitStatus exitStatus );

    void saveSettings();

private:

    void loadSettings();

    QString convertImage(const QImage &img, const QString &format, int screenNum = 0, bool flipflop = false);
    void startNextJob();
    void updateDesktopTips();

    int m_desktopType;
    QProcess m_processRunner;
    QQueue<JobDef> m_cmdQueue;
    JobDef m_currJob;
    QMap<int,QString> m_desktopTips;
    QString m_lastCreatedFile;
};

#endif // _SYSTEMINTEGRATION_H_
