
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *******************************************************************************/

#ifndef _TCPCLIENT_H_
#define _TCPCLIENT_H_

#include <QtCore/QObject>

class QTcpSocket;
class QTimer;
class QFile;

class TcpClient :
    public QObject
{
    Q_OBJECT

public:

    enum States
    {
        INACTIVE,
        CONNECTING,
        NEGOTIATING,
        SENDING,
        FINISHED
    };

    TcpClient(QObject *parent = 0);

    virtual ~TcpClient();

    QString getStateAsString(quint8 state);

signals:

    void stateChanged(quint8 new_state);

private slots:

    void updateClientProgress(qint64 num_bytes);
    void timeout();
    void readMessage();
    void on_socketConnected();
    void on_socketError(int socket_error);
    void on_socketDisconnected();

private:

    //------------------------

    void setState(quint8 new_state);
    void sendFileInformation();
    void sendFile();

    QTcpSocket *m_socket;
    quint8 m_currentState;
    quint16 m_blockSize;
    quint32 m_bytesWritten;
    quint32 m_bytesToWrite;
    quint32 m_fileSize;
    QByteArray m_fileBlock;
    QFile *m_file;

    //timeout variables-------
    quint32 m_tempBytesWritten;
    quint16 m_timeOut;
    quint16 m_timeOutCounter;
    QTimer *m_connectionTimer;
};

#endif // _TCPCLIENT_H_
