
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2012  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *******************************************************************************/

#ifndef _TCPSERVER_H_
#define _TCPSERVER_H_

#include <QtNetwork/QTcpServer>
#include <QtCore/QFile>
#include <QtCore/QTimer>

class TcpServer : public QObject
{
    Q_OBJECT

public:

    enum States
    {
        INACTIVE = 0,
        LISTENING,
        CONNECTING,
        NEGOTIATING,
        RECEIVING,
        FINISHED
    };

    TcpServer(QObject * parent = 0);
    virtual ~TcpServer();

    static QString getStateAsString(quint8 state);
    bool start(quint16 port);
    void stop();
    void abort();
    QString errorString() const;
    void setTimeOut(quint16 time_out);
    QString getLastDir() const;
    void setLastDir(const QString &last_dir);
    void setReverseConnection(bool reverseConnection);
    bool getReverseConnection();

signals:

    void stateChanged(quint8 new_state);

private slots:

    void updateTcpServerProgress();
    void on_server__newConnection();
    void readMessage();
    void on_socketDisconnected();
    void timeout();
    void on_socketConnected();

private:

    void sendAcceptMessage();
    void setState(quint8 new_state);

    QTcpServer *m_server;
    QTcpSocket *m_socket;
    quint8 m_currentState;
    quint16 m_blockSize;
    quint16 m_port;
    quint32 m_fileSize;
    quint32 m_bytesReceived;
    bool m_reverseConnection;

    //timeout variables-------
    quint32 m_tempBytesReceived;
    quint16 m_timeOut;
    quint16 m_timeOutCounter;
    QTimer *m_connectionTimer;

    //------------------------

    QString m_saveFilename;
    QFile *m_file;
    QByteArray m_fileData;
};

#endif // _TCPSERVER_H_
