
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/qdiriterator.h>
#include "WallpaperProvider.h"

#include "LogHandler.h"
#include "MiscFunctions.h"

WallpaperProvider::WallpaperProvider(const QString &dir, int mode, QObject *parent)
    : QObject(parent),
    m_mode(mode)
{
    m_inputList << dir;
    refreshWallpaperList();
}

WallpaperProvider::WallpaperProvider(const QStringList &files, int mode, QObject *parent)
    : QObject(parent),
    m_mode(mode),
    m_inputList(files)
{
    refreshWallpaperList();
}

WallpaperProvider::~WallpaperProvider()
{
}

void WallpaperProvider::refreshWallpaperList()
{
    foreach (const QString &input, m_inputList)
    {
        if (m_mode == MODE_DIRECTORY) // scan directory
        {
            foreach (const QString &dir, m_inputList)
            {
                QStringList files = scanDirectory(dir);
                m_listWallpaper << files;
            }
        }
        else
        {
            if ( QFileInfo(input).isDir() ) // replace the input dir by all the included files
            {
                QStringList files = scanDirectory(input);
                m_listWallpaper << files;
                m_inputList << files; // add the files to the input list
                m_inputList.removeOne(input); // delete the input directory
            }
            else
            {
                if ( QFileInfo(input).isReadable() )
                    m_listWallpaper << input;
            }
        }
    }

    m_inputList.removeDuplicates();
    m_listWallpaper.removeDuplicates ();
}

QStringList WallpaperProvider::scanDirectory(const QString &val)
{
    QDir dir(val);
    QStringList files;

    if ( !dir.exists() )
    {
        LogHandler::getInstance()->reportError( tr( "Cannot find the directory %1",val.toStdString().c_str() ) );
        return files;
    }

    QStringList formats = MiscFunctions::getAvailablesImageFormats().split(" ");

//  QStringList ext;
//  ext << "*.png" << "*.jpg" << "*.bmp" << "*.gif" ;
    QFileInfoList list = dir.entryInfoList(formats);

    foreach (const QFileInfo &info, list)
    {
        files << info.absoluteFilePath();
    }
    return files;
}

bool WallpaperProvider::addInputs(const QStringList &inputs)
{
    bool ok = true;
    foreach (const QString input, inputs)
    ok &= addInput(input);
    return ok;
}

bool WallpaperProvider::addInput(const QString &input)
{
    if (m_inputList.indexOf(input) != -1)
        return false;

    if ( !QFileInfo(input).isReadable() )
        return false;

    if (m_mode == MODE_DIRECTORY)
    {
        scanDirectory(input);
    }
    else
    {
        m_listWallpaper << input;
    }

    return true;
}

bool WallpaperProvider::deleteInput(const QString &input)
{
    m_inputList.removeAll(input);
    if ( m_listWallpaper.removeAll(input) )
    {
        return true;
    }

    return false;
}
