#ifndef _TCPCLIENT_H_
#define _TCPCLIENT_H_

#include <QtCore/QObject>
#include "msgdefinition.h"

class QTcpSocket;
class QTimer;


#define MSG_HEADER_SIZE 8
#define MSG_LEN_OFFSET 4

class TcpClient : public QObject
{
    Q_OBJECT

public:
    TcpClient(QObject *parent = 0);

    virtual ~TcpClient();

    void start(const QString &host, int port);
    void restart();
    void stop();
    void sendRawMessage(const RawMessage &mess);
	void send(const Message& message); //--CB

signals:
    void stateChanged(bool);
	void messageReceived(const QString&);
	void messageEmitted(const QString &);

private slots:
    void timeout();
    void readMessage();
    void on_socketConnected();
    void on_socketError(int socket_error);
    void on_socketDisconnected();

private:
	void initMessageHeader();

    QTcpSocket* m_socket;

	struct 
	{
		char val[MSG_HEADER_SIZE+1];
		int kmp[MSG_HEADER_SIZE];
	} m_header;

	enum {READ_HEADER, READ_PAYLOAD};
	struct 
	{
		char* base;
		char* start;
		int capacity;
		int accept;
		int payload;
		int state;
	} m_buffer;

    QTimer* m_connectionTimer;
    QString m_host;
    int m_port;
};

#endif // _TCPCLIENT_H_
