
#ifndef _TCPSERVER_H_
#define _TCPSERVER_H_

#include <QtNetwork/QTcpServer>
#include <QtCore/QFile>
#include <QtCore/QTimer>

#include "msgdefinition.h"

class TcpServer :
    public QObject
{
    Q_OBJECT

public:

    enum States
    {
        INACTIVE = 0,
        LISTENING,
        CONNECTED
    };

    TcpServer(QObject * parent = 0);
    virtual ~TcpServer();

    static QString getStateAsString(quint8 state);
    bool start(quint16 port);
    void stop();
    void abort();
    QString errorString() const;
    void setTimeOut(quint16 time_out);
    void sendRawMessage(const RawMessage &);

signals:

    void stateChanged(quint8 new_state);
    void messageReceived(const RawMessage &);
//    void messageReceived(const RawMessage &);
//	void messageEmited(const RawMessage &);
	void messageReceived(const QString &);
	void messageEmitted(const QString &);


private slots:

    void serverNewConnection();
    void readMessage();
    void on_socketDisconnected();
    void timeout();
	void sendScheduledMessage();
	void onReceipt();

private:

    void sendAcceptMessage();
    void setState(quint8 new_state);

    QTcpServer *m_server;
    QTcpSocket *m_socket;
    quint8 m_currentState;
    quint16 m_port;
    quint32 m_bytesReceived;
    quint32 m_blockSize;

    //timeout variables-------
    quint32 m_tempBytesReceived;
    quint16 m_timeOut;
    quint16 m_timeOutCounter;
    QTimer *m_connectionTimer;

	QTimer *m_scheduler;
	QList<RawMessage> m_messagesToSend;
	bool m_receivedStarted;
	qint64 m_remainingLength;
	char m_buffer[10000];

};

#endif // _TCPSERVER_H_
